/*
 * Copyright 2010 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * on the rights to use, copy, modify, merge, publish, distribute, sub
 * license, and/or sell copies of the Software, and to permit persons to whom
 * the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Authors:
 *      Jerome Glisse
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "radeon_pci.h"
#include "atom.h"

struct ctx {
    struct radeon_pci       pradeon;
    void                    *bios;
    unsigned                bios_size;
    struct atom_context     *atom_ctx;
};

/* ATOM accessor methods */
static uint32_t cail_pll_read(struct card_info *info, uint32_t reg)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
    return 0;
}

static void cail_pll_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
}

static uint32_t cail_mc_read(struct card_info *info, uint32_t reg)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
    return 0;
}

static void cail_mc_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);
}

static void cail_reg_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    struct ctx *ctx = info->ctx;

    radeon_mmio_wr32(&ctx->pradeon, reg << 2, val);
}

static uint32_t cail_reg_read(struct card_info *info, uint32_t reg)
{
    struct ctx *ctx = info->ctx;

    return radeon_mmio_rd32(&ctx->pradeon, reg << 2);
}

#define R600_BIOS_0_SCRATCH               0x1724
#define R600_BIOS_1_SCRATCH               0x1728
#define R600_BIOS_2_SCRATCH               0x172c
#define R600_BIOS_3_SCRATCH               0x1730
#define R600_BIOS_4_SCRATCH               0x1734
#define R600_BIOS_5_SCRATCH               0x1738
#define R600_BIOS_6_SCRATCH               0x173c
#define R600_BIOS_7_SCRATCH               0x1740

void atom_initialize_bios_scratch_regs(struct ctx *ctx)
{
    uint32_t bios_2_scratch, bios_6_scratch;

    bios_2_scratch = radeon_mmio_rd32(&ctx->pradeon, R600_BIOS_2_SCRATCH);
    bios_6_scratch = radeon_mmio_rd32(&ctx->pradeon, R600_BIOS_6_SCRATCH);

    /* let the bios control the backlight */
    bios_2_scratch &= ~ATOM_S2_VRI_BRIGHT_ENABLE;

    /* tell the bios not to handle mode switching */
    bios_6_scratch |= ATOM_S6_ACC_BLOCK_DISPLAY_SWITCH;

    radeon_mmio_wr32(&ctx->pradeon, R600_BIOS_2_SCRATCH, bios_2_scratch);
    radeon_mmio_wr32(&ctx->pradeon, R600_BIOS_6_SCRATCH, bios_6_scratch);
}

int atombios_init(struct ctx *ctx)
{
    struct card_info atom_card_info = {
        .reg_read       = cail_reg_read,
        .reg_write      = cail_reg_write,
        .ioreg_read     = cail_reg_read,
        .ioreg_write    = cail_reg_write,
        .mc_read        = cail_mc_read,
        .mc_write       = cail_mc_write,
        .pll_read       = cail_pll_read,
        .pll_write      = cail_pll_write,
    };

    atom_card_info.ctx = ctx;
    ctx->atom_ctx = atom_parse(&atom_card_info, ctx->bios);
    if (ctx->atom_ctx == NULL) {
        return -EINVAL;
    }
    atom_initialize_bios_scratch_regs(ctx);
    atom_allocate_fb_scratch(ctx->atom_ctx);
    return 0;
}

void atombios_fini(struct ctx *ctx)
{
    if (ctx->atom_ctx) {
        free(ctx->atom_ctx->scratch);
        free(ctx->atom_ctx);
    }
}

struct log {
    unsigned data[21];
};

#if 0
static struct log logs[] = {
    {{76,
        0x07b40003,0x00000004,0x01fd8030,0x00000000,
        0x01fd6310,0x00000000,0x7cf71ac0,0x00007fff,
        0x7cf719bc,0x00007fff,0x7cf719b0,0x00007fff,
        0x7cf71990,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf719a0,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{76,
        0x00b40000,0x00000004,0x00000000,0x00000000,
        0xe9470010,0x00007ff7,0xe946e158,0x00007ff7,
        0x7cf7096c,0x00007fff,0x01fd6310,0x00000000,
        0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{76,
        0x00bc0000,0x00000004,0x00000000,0x00000000,
        0xe9470010,0x00007ff7,0xe946e158,0x00007ff7,
        0x7cf7096c,0x00007fff,0x01fd6310,0x00000000,
        0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{76,
        0x00b00000,0x00000004,0x7cf70960,0x00007fff,
        0x7cf709e0,0x00007fff,0x01fd4890,0x00000000,
        0x7cf70960,0x00007fff,0x00000004,0x00000000,
        0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{76,
        0x00b80000,0x00000004,0x7cf70960,0x00007fff,
        0x7cf709e0,0x00007fff,0x01fd4890,0x00000000,
        0x7cf70960,0x00007fff,0x00000004,0x00000000,
        0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{35,
        0x00000000,0x00000000,0x01fd3790,0x00000000,
        0x00000000,0x00000000,0x7cf70e30,0x00007fff,
        0xe8ddc0fe,0x00007ff7,0x7cf70e40,0x00007fff,
        0xe8ddbfcf,0x00007ff7,0x7cf70e50,0x00007fff,
        0xe8dd45e1,0x00007ff7,0x7cf70e70,0x00007fff,
    }},
    {{35,
        0x00000001,0x00000001,0x01fd40b0,0x00000000,
        0x00000000,0x00000000,0x7cf70e30,0x00007fff,
        0xe8ddc0fe,0x00007ff7,0x7cf70e40,0x00007fff,
        0xe8ddbfcf,0x00007ff7,0x7cf70e50,0x00007fff,
        0xe8dd45e1,0x00007ff7,0x7cf70e70,0x00007fff,
    }},
    {{12,
        0x00000000,0x00000000,0x010f0000,0x00000000,
        0x00000000,0x00000000,0xe8cb4276,0x00007ff7,
        0xe9470010,0x00007ff7,0x7cf70e30,0x00007fff,
        0x00000001,0x00000000,0x00000000,0x00000000,
        0x7cf70e10,0x00007fff,0xe8ddc05b,0x00007ff7,
    }},
    {{12,
        0x00000000,0x00000000,0x010f0001,0x00000000,
        0x00000000,0x00000000,0xe8cb4276,0x00007ff7,
        0xe9470010,0x00007ff7,0x7cf70e30,0x00007fff,
        0x00000001,0x00000000,0x00000000,0x00000000,
        0x7cf70e10,0x00007fff,0xe8ddc05b,0x00007ff7,
    }},
    {{17,
        0x00213f48,0x00000010,0xe9470010,0x00007ff7,
        0x7cf70810,0x00007fff,0xe946e158,0x00007ff7,
        0x000278d0,0x00000000,0x00024414,0x00000000,
        0x7cf707e0,0x00007fff,0xe8ddc03b,0x00007ff7,
        0x7cf707f0,0x00007fff,0xe8dd46a1,0x00007ff7,
    }},
    {{60,
        0x00004b69,0x7cf708c0,0x00007fff,0x0002f21a,
        0x00000000,0x01fd2bb0,0x00000000,0x7cf708a0,
        0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf708b0,
        0x00007fff,0xe8dd4701,0x00007ff7,0x7cf708f0,
        0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
    }},
    {{12,
        0xe10000ff,0x00000000,0x00000002,0x00000000,
        0x00000000,0x00000000,0xe8ec83ee,0x00007ff7,
        0x00000000,0x00000000,0x7cf70b00,0x00007fff,
        0xe9470010,0x00007ff7,0x0008ca00,0x00000000,
        0x7cf70ae0,0x00007fff,0xe8ddc1ab,0x00007ff7,
    }},
    {{34,
        0x00000100,0x00000000,0x01fd36d0,0x00000000,
        0xe9470010,0x00007ff7,0x7cf70a50,0x00007fff,
        0x00000000,0x00000000,0x7cf70ae0,0x00007fff,
        0x7cf70a20,0x00007fff,0xe8ddc11e,0x00007ff7,
        0x7cf70a30,0x00007fff,0xe8dd4606,0x00007ff7,
    }},
    {{6,
        0x00000000,0x00000000,0x01fd3790,0x00000000,
        0x00000000,0x00000000,0xe9494588,0x00007ff7,
        0x7cf70a40,0x00007fff,0xe8ddc0de,0x00007ff7,
        0x7cf70a50,0x00007fff,0xe8ddc00f,0x00007ff7,
        0x7cf70a60,0x00007fff,0xe8dd4661,0x00007ff7,
    }},
    {{35,
        0x00000000,0x00000000,0x01fd3790,0x00000000,
        0x00000000,0x00000000,0x7cf70a70,0x00007fff,
        0xe8ddc0fe,0x00007ff7,0x7cf70a80,0x00007fff,
        0xe8ddbfcf,0x00007ff7,0x7cf70a90,0x00007fff,
        0xe8dd45e1,0x00007ff7,0x7cf70ab0,0x00007fff,
    }},
    {{60,
        0x00004b69,0x7cf70900,0x00007fff,0x0002f21a,
        0x00000000,0x01fd2bb0,0x00000000,0x7cf708e0,
        0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf708f0,
        0x00007fff,0xe8dd4701,0x00007ff7,0x7cf70930,
        0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
    }},
    {{12,
        0x4c0400ff,0x00000000,0x00000002,0x00000000,
        0x00000000,0x00000000,0x00000000,0x4107c160,
        0x00000000,0x00000000,0x7cf70aa0,0x00007fff,
        0xe9470010,0x00007ff7,0x0002f82c,0x00000000,
        0x7cf70a80,0x00007fff,0xe8ddc1ab,0x00007ff7,
    }},
    {{65,
        0x00000000,0x00000000,0xe9470010,0x00007ff7,
        0x7cf70620,0x00007fff,0xe8ddc09e,0x00007ff7,
        0x7cf70630,0x00007fff,0xe8ddc07b,0x00007ff7,
        0x7cf70640,0x00007fff,0xe8dd4741,0x00007ff7,
        0x7cf70690,0x00007fff,0xe8ececdd,0x00007ff7,
    }},
    {{12,
        0x3a020000,0x02020020,0x10002100,0x00061a80,
        0x00000000,0x00000000,0xe8ececdd,0x00007ff7,
        0xe9470010,0x00007ff7,0x0201afd0,0x00000000,
        0x7cf70ab0,0x00007fff,0x01fd2ef0,0x00000000,
        0x7cf70680,0x00007fff,0xe8ddc05b,0x00007ff7,
    }},
    {{49,
        0x01180780,0x002d0438,0x002c0058,0x00050004,
        0x00000000,0x00000000,0xd7414f05,0x00000033,
        0x7cf70a60,0x00007fff,0x01fd3790,0x00000000,
        0x7cf706a0,0x00007fff,0x00000000,0x00000000,
        0x7cf70680,0x00007fff,0xe8ddc0bb,0x00007ff7,
    }},
    {{35,
        0x00000100,0x00000000,0x01fd3790,0x00000000,
        0x00000000,0x00000000,0x7cf706c0,0x00007fff,
        0xe8ddc0fe,0x00007ff7,0x7cf706d0,0x00007fff,
        0xe8ddbfb2,0x00007ff7,0x7cf706e0,0x00007fff,
        0xe8dd45c1,0x00007ff7,0x7cf70700,0x00007fff,
    }},
    {{40,
        0x00000000,0x00000000,0x00000000,0x00000033,
        0x00000001,0x00007fff,0xe91d03d4,0x00007ff7,
        0xe9470010,0x00007ff7,0x7cf70540,0x00007fff,
        0x7cf70520,0x00007fff,0xe8ddc14b,0x00007ff7,
        0x7cf70530,0x00007fff,0xe8dd4771,0x00007ff7,
    }},
    {{42,
        0x00000300,0x7cf709e0,0x00007fff,0x01fd36d0,
        0x00000000,0xe9470010,0x00007ff7,0x7cf70700,
        0x00007fff,0xe8ddbf9b,0x00007ff7,0x7cf70710,
        0x00007fff,0xe8dd4521,0x00007ff7,0x7cf70b80,
        0x00007fff,0xe8e15b3f,0x00007ff7,0x7cf70a30,
    }},
    {{17,
        0x00216978,0x00000010,0xe9470010,0x00007ff7,
        0x7cf70980,0x00007fff,0xe946e158,0x00007ff7,
        0x00041eb0,0x00000000,0x00024414,0x00000000,
        0x7cf70950,0x00007fff,0xe8ddc03b,0x00007ff7,
        0x7cf70960,0x00007fff,0xe8dd46a1,0x00007ff7,
    }},
    {{65,
        0x00000000,0x00000000,0xe9470010,0x00007ff7,
        0x7cf70920,0x00007fff,0xe8ddc09e,0x00007ff7,
        0x7cf70930,0x00007fff,0xe8ddc07b,0x00007ff7,
        0x7cf70940,0x00007fff,0xe8dd4741,0x00007ff7,
        0x7cf70990,0x00007fff,0xe8ececdd,0x00007ff7,
    }},
    {{12,
        0x3a0200ff,0x0201001b,0x10002100,0x00000000,
        0x00000000,0x00000000,0xe8ececdd,0x00007ff7,
        0xe9470010,0x00007ff7,0x7cf70a90,0x00007fff,
        0x7cf70a60,0x00007fff,0x01fd2ef0,0x00000000,
        0x7cf70980,0x00007fff,0xe8ddc05b,0x00007ff7,
    }},
    {{50,
        0x0f013a02,0x0002020f,0x00000000,0x00000000,
        0x00000000,0x00000000,0xe9470010,0x00007ff7,
        0xe946e158,0x00007ff7,0x7cf70550,0x00007fff,
        0x7cf70558,0x00007fff,0x7cf70b68,0x00007fff,
        0x7cf70510,0x00007fff,0xe8ddc1fb,0x00007ff7,
    }},
    {{76,
        0x01806978,0x00000002,0xe8f27099,0x00007ff7,
        0xe9470010,0x00007ff7,0x01fd4890,0x00000000,
        0xe9499a10,0x00007ff7,0x7cf70500,0x00007fff,
        0x7cf704d0,0x00007fff,0xe8ddbb5b,0x00007ff7,
        0x7cf704e0,0x00007fff,0xe8dd4501,0x00007ff7,
    }},
    {{50,
        0x01003a02,0x0000000f,0x00000000,0x00000000,
        0x00000000,0x00000000,0xe9470010,0x00007ff7,
        0xe946e158,0x00007ff7,0x7cf705a0,0x00007fff,
        0x7cf705a8,0x00007fff,0xe9485440,0x00007ff7,
        0x7cf70560,0x00007fff,0xe8ddc1fb,0x00007ff7,
     }},
    {{17,
         0x00213f48,0x00000010,0xe9470010,0x00007ff7,
         0x7cf707d0,0x00007fff,0xe946e158,0x00007ff7,
         0x000278d0,0x00000000,0x00024414,0x00000000,
         0x7cf707a0,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf707b0,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{60,
         0x00004b69,0x7cf70880,0x00007fff,0x0002f21a,
         0x00000000,0x01fd2bb0,0x00000000,0x7cf70860,
         0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf70870,
         0x00007fff,0xe8dd4701,0x00007ff7,0x7cf708b0,
         0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
     }},
    {{17,
         0x00213f48,0x00000010,0xe9470010,0x00007ff7,
         0x7cf71a00,0x00007fff,0xe946e158,0x00007ff7,
         0x000278d0,0x00000000,0x00024414,0x00000000,
         0x7cf719d0,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf719e0,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{6,
         0x00000100,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0x7cf71e10,0x00007fff,
         0x7cf71d80,0x00007fff,0xe8ddc0de,0x00007ff7,
         0x7cf71d90,0x00007fff,0xe8ddbff2,0x00007ff7,
         0x7cf71da0,0x00007fff,0xe8dd4641,0x00007ff7,
     }},
    {{34,
         0x00000000,0x00000000,0x00000000,0x00000000,
         0x01fd3790,0x00000000,0x7cf71d90,0x00007fff,
         0x00000000,0x00000000,0x00000001,0x00000000,
         0x7cf71d50,0x00007fff,0xe8ddc11e,0x00007ff7,
         0x7cf71d60,0x00007fff,0xe8dd4623,0x00007ff7,
     }},
    {{76,
         0x07b40003,0x00000004,0x01fd8030,0x00000000,
         0x01fd6310,0x00000000,0x7cf71ac0,0x00007fff,
         0x7cf719bc,0x00007fff,0x7cf719b0,0x00007fff,
         0x7cf71990,0x00007fff,0xe8ddbb5b,0x00007ff7,
         0x7cf719a0,0x00007fff,0xe8dd4501,0x00007ff7,
     }},
    {{76,
     }},
    {{76,
         0x00bc0000,0x00000004,0x00000000,0x00000000,
         0xe9470010,0x00007ff7,0xe946e158,0x00007ff7,
         0x7cf7096c,0x00007fff,0x01fd6310,0x00000000,
         0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
         0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
     }},
    {{76,
         0x00b00000,0x00000004,0x7cf70960,0x00007fff,
         0x7cf709e0,0x00007fff,0x01fd4890,0x00000000,
         0x7cf70960,0x00007fff,0x00000004,0x00000000,
         0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
         0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
     }},
    {{76,
         0x00b80000,0x00000004,0x7cf70960,0x00007fff,
         0x7cf709e0,0x00007fff,0x01fd4890,0x00000000,
         0x7cf70960,0x00007fff,0x00000004,0x00000000,
         0x7cf70940,0x00007fff,0xe8ddbb5b,0x00007ff7,
         0x7cf70950,0x00007fff,0xe8dd4501,0x00007ff7,
     }},
    {{35,
         0x00000000,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0x7cf70e30,0x00007fff,
         0xe8ddc0fe,0x00007ff7,0x7cf70e40,0x00007fff,
         0xe8ddbfcf,0x00007ff7,0x7cf70e50,0x00007fff,
         0xe8dd45e1,0x00007ff7,0x7cf70e70,0x00007fff,
     }},
    {{35,
         0x00000001,0x00000001,0x01fd40b0,0x00000000,
         0x00000000,0x00000000,0x7cf70e30,0x00007fff,
         0xe8ddc0fe,0x00007ff7,0x7cf70e40,0x00007fff,
         0xe8ddbfcf,0x00007ff7,0x7cf70e50,0x00007fff,
         0xe8dd45e1,0x00007ff7,0x7cf70e70,0x00007fff,
     }},
    {{12,
         0x00000000,0x00000000,0x010f0000,0x00000000,
         0x00000000,0x00000000,0xe8cb4276,0x00007ff7,
         0xe9470010,0x00007ff7,0x7cf70e30,0x00007fff,
         0x00000001,0x00000000,0x00000000,0x00000000,
         0x7cf70e10,0x00007fff,0xe8ddc05b,0x00007ff7,
     }},
    {{12,
         0x00000000,0x00000000,0x010f0001,0x00000000,
         0x00000000,0x00000000,0xe8cb4276,0x00007ff7,
         0xe9470010,0x00007ff7,0x7cf70e30,0x00007fff,
         0x00000001,0x00000000,0x00000000,0x00000000,
         0x7cf70e10,0x00007fff,0xe8ddc05b,0x00007ff7,
     }},
    {{17,
         0x00213f48,0x00000010,0xe9470010,0x00007ff7,
         0x7cf70810,0x00007fff,0xe946e158,0x00007ff7,
         0x000278d0,0x00000000,0x00024414,0x00000000,
         0x7cf707e0,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf707f0,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{60,
         0x00004b69,0x7cf708c0,0x00007fff,0x0002f21a,
         0x00000000,0x01fd2bb0,0x00000000,0x7cf708a0,
         0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf708b0,
         0x00007fff,0xe8dd4701,0x00007ff7,0x7cf708f0,
         0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
     }},
    {{12,
         0xe10000ff,0x00000000,0x00000002,0x00000000,
         0x00000000,0x00000000,0xe8ec83ee,0x00007ff7,
         0x00000000,0x00000000,0x7cf70b00,0x00007fff,
         0xe9470010,0x00007ff7,0x0008ca00,0x00000000,
         0x7cf70ae0,0x00007fff,0xe8ddc1ab,0x00007ff7,
     }},
    {{34,
         0x00000100,0x00000000,0x01fd36d0,0x00000000,
         0xe9470010,0x00007ff7,0x7cf70a50,0x00007fff,
         0x00000000,0x00000000,0x7cf70ae0,0x00007fff,
         0x7cf70a20,0x00007fff,0xe8ddc11e,0x00007ff7,
         0x7cf70a30,0x00007fff,0xe8dd4606,0x00007ff7,
     }},
    {{6,
         0x00000000,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0xe9494588,0x00007ff7,
         0x7cf70a40,0x00007fff,0xe8ddc0de,0x00007ff7,
         0x7cf70a50,0x00007fff,0xe8ddc00f,0x00007ff7,
         0x7cf70a60,0x00007fff,0xe8dd4661,0x00007ff7,
     }},
    {{35,
         0x00000000,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0x7cf70a70,0x00007fff,
         0xe8ddc0fe,0x00007ff7,0x7cf70a80,0x00007fff,
         0xe8ddbfcf,0x00007ff7,0x7cf70a90,0x00007fff,
         0xe8dd45e1,0x00007ff7,0x7cf70ab0,0x00007fff,
     }},
    {{60,
         0x00004b69,0x7cf70900,0x00007fff,0x0002f21a,
         0x00000000,0x01fd2bb0,0x00000000,0x7cf708e0,
         0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf708f0,
         0x00007fff,0xe8dd4701,0x00007ff7,0x7cf70930,
         0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
     }},
    {{12,
         0x4c0400ff,0x00000000,0x00000002,0x00000000,
         0x00000000,0x00000000,0x00000000,0x4107c160,
         0x00000000,0x00000000,0x7cf70aa0,0x00007fff,
         0xe9470010,0x00007ff7,0x0002f82c,0x00000000,
         0x7cf70a80,0x00007fff,0xe8ddc1ab,0x00007ff7,
     }},
    {{65,
         0x00000000,0x00000000,0xe9470010,0x00007ff7,
         0x7cf70620,0x00007fff,0xe8ddc09e,0x00007ff7,
         0x7cf70630,0x00007fff,0xe8ddc07b,0x00007ff7,
         0x7cf70640,0x00007fff,0xe8dd4741,0x00007ff7,
         0x7cf70690,0x00007fff,0xe8ececdd,0x00007ff7,
     }},
    {{12,
         0x3a020000,0x02020020,0x10002100,0x00061a80,
         0x00000000,0x00000000,0xe8ececdd,0x00007ff7,
         0xe9470010,0x00007ff7,0x0201afd0,0x00000000,
         0x7cf70ab0,0x00007fff,0x01fd2ef0,0x00000000,
         0x7cf70680,0x00007fff,0xe8ddc05b,0x00007ff7,
     }},
    {{49,
         0x01180780,0x002d0438,0x002c0058,0x00050004,
         0x00000000,0x00000000,0xd7414f05,0x00000033,
         0x7cf70a60,0x00007fff,0x01fd3790,0x00000000,
         0x7cf706a0,0x00007fff,0x00000000,0x00000000,
         0x7cf70680,0x00007fff,0xe8ddc0bb,0x00007ff7,
     }},
    {{35,
         0x00000100,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0x7cf706c0,0x00007fff,
         0xe8ddc0fe,0x00007ff7,0x7cf706d0,0x00007fff,
         0xe8ddbfb2,0x00007ff7,0x7cf706e0,0x00007fff,
         0xe8dd45c1,0x00007ff7,0x7cf70700,0x00007fff,
     }},
    {{40,
         0x00000000,0x00000000,0x00000000,0x00000033,
         0x00000001,0x00007fff,0xe91d03d4,0x00007ff7,
         0xe9470010,0x00007ff7,0x7cf70540,0x00007fff,
         0x7cf70520,0x00007fff,0xe8ddc14b,0x00007ff7,
         0x7cf70530,0x00007fff,0xe8dd4771,0x00007ff7,
     }},
    {{42,
         0x00000300,0x7cf709e0,0x00007fff,0x01fd36d0,
         0x00000000,0xe9470010,0x00007ff7,0x7cf70700,
         0x00007fff,0xe8ddbf9b,0x00007ff7,0x7cf70710,
         0x00007fff,0xe8dd4521,0x00007ff7,0x7cf70b80,
         0x00007fff,0xe8e15b3f,0x00007ff7,0x7cf70a30,
     }},
    {{17,
         0x00216978,0x00000010,0xe9470010,0x00007ff7,
         0x7cf70980,0x00007fff,0xe946e158,0x00007ff7,
         0x00041eb0,0x00000000,0x00024414,0x00000000,
         0x7cf70950,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf70960,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{65,
         0x00000000,0x00000000,0xe9470010,0x00007ff7,
         0x7cf70920,0x00007fff,0xe8ddc09e,0x00007ff7,
         0x7cf70930,0x00007fff,0xe8ddc07b,0x00007ff7,
         0x7cf70940,0x00007fff,0xe8dd4741,0x00007ff7,
         0x7cf70990,0x00007fff,0xe8ececdd,0x00007ff7,
     }},
    {{12,
         0x3a0200ff,0x0201001b,0x10002100,0x00000000,
         0x00000000,0x00000000,0xe8ececdd,0x00007ff7,
         0xe9470010,0x00007ff7,0x7cf70a90,0x00007fff,
         0x7cf70a60,0x00007fff,0x01fd2ef0,0x00000000,
         0x7cf70980,0x00007fff,0xe8ddc05b,0x00007ff7,
     }},
    {{50,
         0x0f013a02,0x0002020f,0x00000000,0x00000000,
         0x00000000,0x00000000,0xe9470010,0x00007ff7,
         0xe946e158,0x00007ff7,0x7cf70550,0x00007fff,
         0x7cf70558,0x00007fff,0x7cf70b68,0x00007fff,
         0x7cf70510,0x00007fff,0xe8ddc1fb,0x00007ff7,
     }},
    {{76,
         0x01806978,0x00000002,0xe8f27099,0x00007ff7,
         0xe9470010,0x00007ff7,0x01fd4890,0x00000000,
         0xe9499a10,0x00007ff7,0x7cf70500,0x00007fff,
         0x7cf704d0,0x00007fff,0xe8ddbb5b,0x00007ff7,
         0x7cf704e0,0x00007fff,0xe8dd4501,0x00007ff7,
     }},
    {{50,
         0x01003a02,0x0000000f,0x00000000,0x00000000,
         0x00000000,0x00000000,0xe9470010,0x00007ff7,
         0xe946e158,0x00007ff7,0x7cf705a0,0x00007fff,
         0x7cf705a8,0x00007fff,0xe9485440,0x00007ff7,
         0x7cf70560,0x00007fff,0xe8ddc1fb,0x00007ff7,
     }},
    {{17,
         0x00213f48,0x00000010,0xe9470010,0x00007ff7,
         0x7cf707d0,0x00007fff,0xe946e158,0x00007ff7,
         0x000278d0,0x00000000,0x00024414,0x00000000,
         0x7cf707a0,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf707b0,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{60,
         0x00004b69,0x7cf70880,0x00007fff,0x0002f21a,
         0x00000000,0x01fd2bb0,0x00000000,0x7cf70860,
         0x00007fff,0xe8ddc1cb,0x00007ff7,0x7cf70870,
         0x00007fff,0xe8dd4701,0x00007ff7,0x7cf708b0,
         0x00007fff,0xe8ecd99b,0x00007ff7,0x0002f21a,
     }},
    {{17,
         0x00213f48,0x00000010,0xe9470010,0x00007ff7,
         0x7cf71a00,0x00007fff,0xe946e158,0x00007ff7,
         0x000278d0,0x00000000,0x00024414,0x00000000,
         0x7cf719d0,0x00007fff,0xe8ddc03b,0x00007ff7,
         0x7cf719e0,0x00007fff,0xe8dd46a1,0x00007ff7,
     }},
    {{6,
         0x00000100,0x00000000,0x01fd3790,0x00000000,
         0x00000000,0x00000000,0x7cf71e10,0x00007fff,
         0x7cf71d80,0x00007fff,0xe8ddc0de,0x00007ff7,
         0x7cf71d90,0x00007fff,0xe8ddbff2,0x00007ff7,
         0x7cf71da0,0x00007fff,0xe8dd4641,0x00007ff7,
     }},
    {{34,
         0x00000000,0x00000000,0x00000000,0x00000000,
         0x01fd3790,0x00000000,0x7cf71d90,0x00007fff,
         0x00000000,0x00000000,0x00000001,0x00000000,
         0x7cf71d50,0x00007fff,0xe8ddc11e,0x00007ff7,
         0x7cf71d60,0x00007fff,0xe8dd4623,0x00007ff7,
     }},
};

void replay_f(struct atom_context *atom_ctx)
{
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 3;
	args.usInitInfo = CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_D;
	args.asMode.ucLaneSel = 3;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 1;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_INIT;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 1;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 3;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 2;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 1;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 1;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189632;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 57600;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189696;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}

// ---------------------------------------
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 19460;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 32;
	args.ucPostDiv = 2;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 400000;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	SET_CRTC_USING_DTD_TIMING_PARAMETERS args;
	args.usH_Size = 1920;
	args.usH_Blanking_Time = 280;
	args.usV_Size = 1080;
	args.usV_Blanking_Time = 45;
	args.usH_SyncOffset = 88;
	args.usH_SyncWidth = 44;
	args.usV_SyncOffset = 4;
	args.usV_SyncWidth = 5;
	args.susModeMiscInfo.sbfAccess.HorizontalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.HSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VerticalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.H_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.V_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.CompositeSync = 0;
	args.susModeMiscInfo.sbfAccess.Interlace = 0;
	args.susModeMiscInfo.sbfAccess.DoubleClock = 0;
	args.susModeMiscInfo.sbfAccess.RGB888 = 0;
	args.susModeMiscInfo.usAccess = 0x0;
	args.ucH_Border = 0;
	args.ucV_Border = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 49, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	SET_CRTC_OVERSCAN_PARAMETERS args;
	args.usOverscanRight = 0;
	args.usOverscanLeft = 0;
	args.usOverscanBottom = 0;
	args.usOverscanTop = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 40, (uint32_t*)&args);
}
{
	SELECT_CRTC_SOURCE_PARAMETERS_V2 args;
	args.ucCRTC = 0;
	args.ucEncoderID = ASIC_INT_DIG1_ENCODER_ID;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	atom_execute_table(atom_ctx, 42, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 27;
	args.ucPostDiv = 1;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x1;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENCODER_SETUP;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 2;
	args.ucBitPerColor = 2;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.usInitInfo = 27000;
	args.asMode.ucLaneSel = 120;
	args.asMode.ucLaneSet = 105;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 0;
	args.acConfig.ucEncoderSel = 0;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_ENABLE;
	args.ucLaneNum = 2;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x0;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENABLE_OUTPUT;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 0;
	args.ucBitPerColor = 0;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189568;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING_OFF;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
// ---------------------------------------
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 3;
	args.usInitInfo = CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_D;
	args.asMode.ucLaneSel = 3;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 1;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_INIT;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 0;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 0;
	args.acConfig.ucEncoderSel = 0;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 0;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 3;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 2;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 1;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 1;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189632;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 57600;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189696;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 19460;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 32;
	args.ucPostDiv = 2;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 400000;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	SET_CRTC_USING_DTD_TIMING_PARAMETERS args;
	args.usH_Size = 1920;
	args.usH_Blanking_Time = 280;
	args.usV_Size = 1080;
	args.usV_Blanking_Time = 45;
	args.usH_SyncOffset = 88;
	args.usH_SyncWidth = 44;
	args.usV_SyncOffset = 4;
	args.usV_SyncWidth = 5;
	args.susModeMiscInfo.sbfAccess.HorizontalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.HSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VerticalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.H_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.V_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.CompositeSync = 0;
	args.susModeMiscInfo.sbfAccess.Interlace = 0;
	args.susModeMiscInfo.sbfAccess.DoubleClock = 0;
	args.susModeMiscInfo.sbfAccess.RGB888 = 0;
	args.susModeMiscInfo.usAccess = 0x0;
	args.ucH_Border = 0;
	args.ucV_Border = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 49, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	SET_CRTC_OVERSCAN_PARAMETERS args;
	args.usOverscanRight = 0;
	args.usOverscanLeft = 0;
	args.usOverscanBottom = 0;
	args.usOverscanTop = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 40, (uint32_t*)&args);
}
{
	SELECT_CRTC_SOURCE_PARAMETERS_V2 args;
	args.ucCRTC = 0;
	args.ucEncoderID = ASIC_INT_DIG1_ENCODER_ID;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	atom_execute_table(atom_ctx, 42, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 27;
	args.ucPostDiv = 1;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x1;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENCODER_SETUP;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 2;
	args.ucBitPerColor = 2;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.usInitInfo = 27000;
	args.asMode.ucLaneSel = 120;
	args.asMode.ucLaneSet = 105;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 0;
	args.acConfig.ucEncoderSel = 0;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_ENABLE;
	args.ucLaneNum = 2;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x0;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENABLE_OUTPUT;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 0;
	args.ucBitPerColor = 0;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 16189568;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING_OFF;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
}
#else
static struct log logs[] = {
{{76,
0x07b40003,0x00000004,0x00f89030,0x00000000,
0x00f87310,0x00000000,0x065670b0,0x00007fff,
0x06566fac,0x00007fff,0x06566fa0,0x00007fff,
0x06566f80,0x00007fff,0x7de75b5b,0x00007ff2,
0x06566f90,0x00007fff,0x7de6e501,0x00007ff2}},
{{76,
0x00b40000,0x00000004,0x00000000,0x00000000,
0x7e50a010,0x00007ff2,0x7e508158,0x00007ff2,
0x06565f5c,0x00007fff,0x00f87310,0x00000000,
0x06565f30,0x00007fff,0x7de75b5b,0x00007ff2,
0x06565f40,0x00007fff,0x7de6e501,0x00007ff2}},
{{76,
0x00bc0000,0x00000004,0x00000000,0x00000000,
0x7e50a010,0x00007ff2,0x7e508158,0x00007ff2,
0x06565f5c,0x00007fff,0x00f87310,0x00000000,
0x06565f30,0x00007fff,0x7de75b5b,0x00007ff2,
0x06565f40,0x00007fff,0x7de6e501,0x00007ff2}},
{{76,
0x00b00000,0x00000004,0x06565f50,0x00007fff,
0x06565fd0,0x00007fff,0x00f85890,0x00000000,
0x06565f50,0x00007fff,0x00000004,0x00000000,
0x06565f30,0x00007fff,0x7de75b5b,0x00007ff2,
0x06565f40,0x00007fff,0x7de6e501,0x00007ff2}},
{{76,
0x00b80000,0x00000004,0x06565f50,0x00007fff,
0x06565fd0,0x00007fff,0x00f85890,0x00000000,
0x06565f50,0x00007fff,0x00000004,0x00000000,
0x06565f30,0x00007fff,0x7de75b5b,0x00007ff2,
0x06565f40,0x00007fff,0x7de6e501,0x00007ff2}},
{{35,
0x00000000,0x00000000,0x00f84790,0x00000000,
0x00000000,0x00000000,0x06566420,0x00007fff,
0x7de760fe,0x00007ff2,0x06566430,0x00007fff,
0x7de75fcf,0x00007ff2,0x06566440,0x00007fff,
0x7de6e5e1,0x00007ff2,0x06566460,0x00007fff}},
{{35,
0x00000001,0x00000001,0x00f850b0,0x00000000,
0x00000000,0x00000000,0x06566420,0x00007fff,
0x7de760fe,0x00007ff2,0x06566430,0x00007fff,
0x7de75fcf,0x00007ff2,0x06566440,0x00007fff,
0x7de6e5e1,0x00007ff2,0x06566460,0x00007fff}},
{{12,
0x00000000,0x00000000,0x010f0000,0x00000000,
0x00000000,0x00000000,0x7dd4e276,0x00007ff2,
0x7e50a010,0x00007ff2,0x06566420,0x00007fff,
0x00000001,0x00000000,0x00000000,0x00000000,
0x06566400,0x00007fff,0x7de7605b,0x00007ff2}},
{{12,
0x00000000,0x00000000,0x010f0001,0x00000000,
0x00000000,0x00000000,0x7dd4e276,0x00007ff2,
0x7e50a010,0x00007ff2,0x06566420,0x00007fff,
0x00000001,0x00000000,0x00000000,0x00000000,
0x06566400,0x00007fff,0x7de7605b,0x00007ff2}},
{{17,
0x00213f48,0x00000010,0x7e50a010,0x00007ff2,
0x06565e00,0x00007fff,0x7e508158,0x00007ff2,
0x000278d0,0x00000000,0x00024414,0x00000000,
0x06565dd0,0x00007fff,0x7de7603b,0x00007ff2,
0x06565de0,0x00007fff,0x7de6e6a1,0x00007ff2}},
{{60,
0x00004b69,0x06565eb0,0x00007fff,0x0002f21a,
0x00000000,0x00f83bb0,0x00000000,0x06565e90,
0x00007fff,0x7de761cb,0x00007ff2,0x06565ea0,
0x00007fff,0x7de6e701,0x00007ff2,0x06565ee0,
0x00007fff,0x7df6799b,0x00007ff2,0x0002f21a}},
{{12,
0xe10000ff,0x00000000,0x00000002,0x00000000,
0x00000000,0x00000000,0x7df623ee,0x00007ff2,
0x00000000,0x00000000,0x065660f0,0x00007fff,
0x7e50a010,0x00007ff2,0x0008ca00,0x00000000,
0x065660d0,0x00007fff,0x7de761ab,0x00007ff2}},
{{34,
0x00000100,0x00000000,0x00f846d0,0x00000000,
0x7e50a010,0x00007ff2,0x06566040,0x00007fff,
0x00000000,0x00000000,0x065660d0,0x00007fff,
0x06566010,0x00007fff,0x7de7611e,0x00007ff2,
0x06566020,0x00007fff,0x7de6e606,0x00007ff2}},
{{6,
0x00000000,0x00000000,0x00f84790,0x00000000,
0x00000000,0x00000000,0x7e52e588,0x00007ff2,
0x06566030,0x00007fff,0x7de760de,0x00007ff2,
0x06566040,0x00007fff,0x7de7600f,0x00007ff2,
0x06566050,0x00007fff,0x7de6e661,0x00007ff2}},
{{35,
0x00000000,0x00000000,0x00f84790,0x00000000,
0x00000000,0x00000000,0x06566060,0x00007fff,
0x7de760fe,0x00007ff2,0x06566070,0x00007fff,
0x7de75fcf,0x00007ff2,0x06566080,0x00007fff,
0x7de6e5e1,0x00007ff2,0x065660a0,0x00007fff}},
{{60,
0x00004b69,0x06565ef0,0x00007fff,0x0002f21a,
0x00000000,0x00f83bb0,0x00000000,0x06565ed0,
0x00007fff,0x7de761cb,0x00007ff2,0x06565ee0,
0x00007fff,0x7de6e701,0x00007ff2,0x06565f20,
0x00007fff,0x7df6799b,0x00007ff2,0x0002f21a}},
{{12,
0x4c0400ff,0x00000000,0x00000002,0x00000000,
0x00000000,0x00000000,0x00000000,0x4107c160,
0x00000000,0x00000000,0x06566090,0x00007fff,
0x7e50a010,0x00007ff2,0x0002f82c,0x00000000,
0x06566070,0x00007fff,0x7de761ab,0x00007ff2}},
{{65,
0x00000000,0x00000000,0x7e50a010,0x00007ff2,
0x06565c10,0x00007fff,0x7de7609e,0x00007ff2,
0x06565c20,0x00007fff,0x7de7607b,0x00007ff2,
0x06565c30,0x00007fff,0x7de6e741,0x00007ff2,
0x06565c80,0x00007fff,0x7df68cdd,0x00007ff2}},
{{12,
0x3a020000,0x02020020,0x10002100,0x00061a80,
0x00000000,0x00000000,0x7df68cdd,0x00007ff2,
0x7e50a010,0x00007ff2,0x00fcbfd0,0x00000000,
0x065660a0,0x00007fff,0x00f83ef0,0x00000000,
0x06565c70,0x00007fff,0x7de7605b,0x00007ff2}},
{{49,
0x01180780,0x002d0438,0x002c0058,0x00050004,
0x00000000,0x00000000,0xd7414f05,0x00000033,
0x06566050,0x00007fff,0x00f84790,0x00000000,
0x06565c90,0x00007fff,0x00000000,0x00000000,
0x06565c70,0x00007fff,0x7de760bb,0x00007ff2}},
{{35,
0x00000100,0x00000000,0x00f84790,0x00000000,
0x00000000,0x00000000,0x06565cb0,0x00007fff,
0x7de760fe,0x00007ff2,0x06565cc0,0x00007fff,
0x7de75fb2,0x00007ff2,0x06565cd0,0x00007fff,
0x7de6e5c1,0x00007ff2,0x06565cf0,0x00007fff}},
{{40,
0x00000000,0x00000000,0x00000000,0x00000033,
0x00000001,0x00007fff,0x7e26a3d4,0x00007ff2,
0x7e50a010,0x00007ff2,0x06565b30,0x00007fff,
0x06565b10,0x00007fff,0x7de7614b,0x00007ff2,
0x06565b20,0x00007fff,0x7de6e771,0x00007ff2}},
{{42,
0x00000300,0x06565fd0,0x00007fff,0x00f846d0,
0x00000000,0x7e50a010,0x00007ff2,0x06565cf0,
0x00007fff,0x7de75f9b,0x00007ff2,0x06565d00,
0x00007fff,0x7de6e521,0x00007ff2,0x06566170,
0x00007fff,0x7deafb3f,0x00007ff2,0x06566020}},
{{17,
0x00216978,0x00000010,0x7e50a010,0x00007ff2,
0x06565f70,0x00007fff,0x7e508158,0x00007ff2,
0x00041eb0,0x00000000,0x00024414,0x00000000,
0x06565f40,0x00007fff,0x7de7603b,0x00007ff2,
0x06565f50,0x00007fff,0x7de6e6a1,0x00007ff2}},
{{65,
0x00000000,0x00000000,0x7e50a010,0x00007ff2,
0x06565f10,0x00007fff,0x7de7609e,0x00007ff2,
0x06565f20,0x00007fff,0x7de7607b,0x00007ff2,
0x06565f30,0x00007fff,0x7de6e741,0x00007ff2,
0x06565f80,0x00007fff,0x7df68cdd,0x00007ff2}},
{{12,
0x3a0200ff,0x0201001b,0x10002100,0x00000000,
0x00000000,0x00000000,0x7df68cdd,0x00007ff2,
0x7e50a010,0x00007ff2,0x06566080,0x00007fff,
0x06566050,0x00007fff,0x00f83ef0,0x00000000,
0x06565f70,0x00007fff,0x7de7605b,0x00007ff2}},
{{50,
0x0f013a02,0x0002020f,0x00000000,0x00000000,
0x00000000,0x00000000,0x7e50a010,0x00007ff2,
0x7e508158,0x00007ff2,0x06565b40,0x00007fff,
0x06565b48,0x00007fff,0x06566158,0x00007fff,
0x06565b00,0x00007fff,0x7de761fb,0x00007ff2}},
{{76,
0x01806978,0x00000002,0x7dfc1099,0x00007ff2,
0x7e50a010,0x00007ff2,0x00f85890,0x00000000,
0x7e533a10,0x00007ff2,0x06565af0,0x00007fff,
0x06565ac0,0x00007fff,0x7de75b5b,0x00007ff2,
0x06565ad0,0x00007fff,0x7de6e501,0x00007ff2}},
{{50,
0x01003a02,0x0000000f,0x00000000,0x00000000,
0x00000000,0x00000000,0x7e50a010,0x00007ff2,
0x7e508158,0x00007ff2,0x06565b90,0x00007fff,
0x06565b98,0x00007fff,0x7e51f440,0x00007ff2,
0x06565b50,0x00007fff,0x7de761fb,0x00007ff2}},
{{17,
0x00213f48,0x00000010,0x7e50a010,0x00007ff2,
0x06565dc0,0x00007fff,0x7e508158,0x00007ff2,
0x000278d0,0x00000000,0x00024414,0x00000000,
0x06565d90,0x00007fff,0x7de7603b,0x00007ff2,
0x06565da0,0x00007fff,0x7de6e6a1,0x00007ff2}},
{{60,
0x00004b69,0x06565e70,0x00007fff,0x0002f21a,
0x00000000,0x00f83bb0,0x00000000,0x06565e50,
0x00007fff,0x7de761cb,0x00007ff2,0x06565e60,
0x00007fff,0x7de6e701,0x00007ff2,0x06565ea0,
0x00007fff,0x7df6799b,0x00007ff2,0x0002f21a}},
{{17,
0x00213f48,0x00000010,0x7e50a010,0x00007ff2,
0x06566ff0,0x00007fff,0x7e508158,0x00007ff2,
0x000278d0,0x00000000,0x00024414,0x00000000,
0x06566fc0,0x00007fff,0x7de7603b,0x00007ff2,
0x06566fd0,0x00007fff,0x7de6e6a1,0x00007ff2}},
{{6,
0x00000100,0x00000000,0x00f84790,0x00000000,
0x00000000,0x00000000,0x06567400,0x00007fff,
0x06567370,0x00007fff,0x7de760de,0x00007ff2,
0x06567380,0x00007fff,0x7de75ff2,0x00007ff2,
0x06567390,0x00007fff,0x7de6e641,0x00007ff2}},
{{34,
0x00000000,0x00000000,0x00000000,0x00000000,
0x00f84790,0x00000000,0x06567380,0x00007fff,
0x00000000,0x00000000,0x00000001,0x00000000,
0x06567340,0x00007fff,0x7de7611e,0x00007ff2,
0x06567350,0x00007fff,0x7de6e623,0x00007ff2}},
};

void replay_f(struct atom_context *atom_ctx)
{
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 3;
	args.usInitInfo = CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_D;
	args.asMode.ucLaneSel = 3;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 1;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_INIT;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 1;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 3;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 0;
	args.usInitInfo = CONNECTOR_OBJECT_ID_NONE;
	args.asMode.ucLaneSel = 0;
	args.asMode.ucLaneSet = 0;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 1;
	args.acConfig.ucEncoderSel = 1;
	args.acConfig.ucRefClkSource = 2;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_DISABLE;
	args.ucLaneNum = 4;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 1;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 0;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 1;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 15;
	args.ucMiscInfo = 0x1;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 5660336;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 57600;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 0;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 5660400;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 19460;
	args.usFbDiv = 0;
	args.ucPostDiv = 0;
	args.ucRefDiv = 0;
	args.ucRefDiv = 0;
	args.ucPpll = 2;
	args.ucTransmitterID = ENCODER_OBJECT_ID_NONE;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x0;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 0;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 32;
	args.ucPostDiv = 2;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 400000;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	SET_CRTC_USING_DTD_TIMING_PARAMETERS args;
	args.usH_Size = 1920;
	args.usH_Blanking_Time = 280;
	args.usV_Size = 1080;
	args.usV_Blanking_Time = 45;
	args.usH_SyncOffset = 88;
	args.usH_SyncWidth = 44;
	args.usV_SyncOffset = 4;
	args.usV_SyncWidth = 5;
	args.susModeMiscInfo.sbfAccess.HorizontalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.HSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VSyncPolarity = 0;
	args.susModeMiscInfo.sbfAccess.VerticalCutOff = 0;
	args.susModeMiscInfo.sbfAccess.H_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.V_ReplicationBy2 = 0;
	args.susModeMiscInfo.sbfAccess.CompositeSync = 0;
	args.susModeMiscInfo.sbfAccess.Interlace = 0;
	args.susModeMiscInfo.sbfAccess.DoubleClock = 0;
	args.susModeMiscInfo.sbfAccess.RGB888 = 0;
	args.susModeMiscInfo.usAccess = 0x0;
	args.ucH_Border = 0;
	args.ucV_Border = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 49, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 35, (uint32_t*)&args);
}
{
	SET_CRTC_OVERSCAN_PARAMETERS args;
	args.usOverscanRight = 0;
	args.usOverscanLeft = 0;
	args.usOverscanBottom = 0;
	args.usOverscanTop = 0;
	args.ucCRTC = 0;
	atom_execute_table(atom_ctx, 40, (uint32_t*)&args);
}
{
	SELECT_CRTC_SOURCE_PARAMETERS_V2 args;
	args.ucCRTC = 0;
	args.ucEncoderID = ASIC_INT_DIG1_ENCODER_ID;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	atom_execute_table(atom_ctx, 42, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_LVDS_SS_PARAMETERS_V2 args;
	args.usSpreadSpectrumPercentage = 0;
	args.ucSpreadSpectrumType = 0x0;
	args.ucSpreadSpectrumStep = 0;
	args.ucEnable = 0;
	args.ucSpreadSpectrumDelay = 0;
	args.ucSpreadSpectrumRange = 0;
	atom_execute_table(atom_ctx, 65, (uint32_t*)&args);
}
{
	PIXEL_CLOCK_PARAMETERS_V5 args;
	args.ucCRTC = 255;
	args.ucReserved = 0;
	args.ucFracFbDiv = 0;
	args.usPixelClock = 14850;
	args.usFbDiv = 27;
	args.ucPostDiv = 1;
	args.ucRefDiv = 2;
	args.ucRefDiv = 2;
	args.ucPpll = 0;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncoderMode = 0;
	args.ucMiscInfo = 0x10;
	args.ulFbDivDecFrac = 0;
	atom_execute_table(atom_ctx, 12, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x1;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENCODER_SETUP;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 2;
	args.ucBitPerColor = 2;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 27000;
	args.usInitInfo = 27000;
	args.asMode.ucLaneSel = 120;
	args.asMode.ucLaneSet = 105;
	args.acConfig.fDualLinkConnector = 1;
	args.acConfig.fCoherentMode = 0;
	args.acConfig.ucLinkSel = 0;
	args.acConfig.ucEncoderSel = 0;
	args.acConfig.ucRefClkSource = 0;
	args.acConfig.ucTransmitterSel = 0;
	args.ucAction = ATOM_TRANSMITTER_ACTION_ENABLE;
	args.ucLaneNum = 2;
	atom_execute_table(atom_ctx, 76, (uint32_t*)&args);
}
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;
	args.usPixelClock = 14850;
	args.usConnectorId = 14850;
	args.ucConfig = 0x0;
	args.ucAction = EXTERNAL_ENCODER_ACTION_V3_ENABLE_OUTPUT;
	args.ucEncoderMode = ATOM_ENCODER_MODE_CRT;
	args.ucLaneNum = 0;
	args.ucBitPerColor = 0;
	atom_execute_table(atom_ctx, 50, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;
	args.ulClock = 5660272;
	args.ucPostDiv = 19305;
	atom_execute_table(atom_ctx, 60, (uint32_t*)&args);
}
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;
	args.usPixelClock = 16200;
	args.ucTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	args.ucEncodeMode = ATOM_ENCODER_MODE_DP;
	args.ucDispPllConfig = 0x10;
	args.ucExtTransmitterID = ENCODER_OBJECT_ID_INTERNAL_UNIPHY2;
	atom_execute_table(atom_ctx, 17, (uint32_t*)&args);
}
{
	ENABLE_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucEnable = 1;
	atom_execute_table(atom_ctx, 6, (uint32_t*)&args);
}
{
	BLANK_CRTC_PARAMETERS args;
	args.ucCRTC = 0;
	args.ucBlanking = ATOM_BLANKING_OFF;
	args.usBlackColorRCr = 0;
	args.usBlackColorGY = 0;
	args.usBlackColorBCb = 0;
	atom_execute_table(atom_ctx, 34, (uint32_t*)&args);
}
}
#endif

int main(int argc, char *argv[])
{
    struct ctx ctx;
    unsigned i;
    int r;

    ctx.bios_size = 64 << 10;
    ctx.bios = calloc(1, ctx.bios_size);
    if (ctx.bios == NULL) {
        return -ENOMEM;
    }
    if (radeon_pci_init(&ctx.pradeon)) {
        return -1;
    }
    r = pci_device_read_rom(ctx.pradeon.dev, ctx.bios);
    if (r) {
        goto out;
    }
    r = atombios_init(&ctx);
    if (r) {
        goto out;
    }

#if 1
    fprintf(stderr, "ASIC INIT --------------------------\n");
    r = atom_asic_init(ctx.atom_ctx);
    if (r) {
        fprintf(stderr, "asic init failed\n");
        goto out;
    }
    fprintf(stderr, "ASIC REPLAY ------------------------\n");
    replay_f(ctx.atom_ctx);
#else
    for (i = 0; i < (sizeof(logs)/sizeof(logs[0])); i++) {
        atom_trace_cmd_header(ctx.atom_ctx, logs[i].data[0], &logs[i].data[1]);
    }
#endif


out:
    atombios_fini(&ctx);
    radeon_pci_fini(&ctx.pradeon);
    return r;
}
